#!/bin/sh

# return absolute path to main script
# SCRIPTPATH=`dirname $0`

# outpus
_QN_RES_FILE=resources.lst
_QN_SHARES_FILE=shares.lst
_QN_HEALTH_FILE=health.lst

# $0 $1 (resource or folders)
qn_main_exec() {
    case $1 in
        resources )
            resources
            ;;
        folders )
            folders $1 $2
            ;;
        health )
            getHealthTest
            ;;
        * )
            echo "Usage: $0 (folders|resources|health)" >&2
            exit 1
    esac
}

# Report a NORMAL state of the passive health.
# Data will be written to _QN_HEALTH_FILE. File will be replaced each time!
# Arguments: NO
ReportHealthNormal() {
    echo '{"state": "NORMAL"}' > "${_QN_HEALTH_FILE}"
}

# Error code for general/unknown problem when report passive health
QN_ERR_CODE_UNKNOWN="UNKNOWN"
# Error code for problem with Firewall when report passive health
QN_ERR_CODE_FIREWALL="FIREWALL"

# Arguments:
#   $1: Error code. If not provided - defaults to QN_ERR_CODE_UNKNOWN.
#       Use ONLY predefined codes like "QN_ERR_CODE_FIREWALL".
#   $2: Error detailed message. If not provided - defaults to "Unknown health error"
#       Use ONLY JSON escaped strings.
# Example: ReportHealthError ${QN_ERR_CODE_FIREWALL} "Cannot open 4090 port"
ReportHealthError() {
    local _code="${1}"
    local _msg="${2}"
    if [ -z "${_code}" ]; then
        _code="${QN_ERR_CODE_UNKNOWN}"
    fi
    if [ -z "${_msg}" ]; then
        _msg="Unknown health error"
    fi
	# TODO Msg is not escaped
    echo "{\"state\": \"REINSTALL_REQUIRED\",\"code\": \"${_code}\",\"details\": \"${_msg}\"}" > "${_QN_HEALTH_FILE}"
}

# Get firewall status. Some nas devices hasn't command line fw implementation. We just write positive status to file.
# On devices where fw can be managed via CL, same func must be added in psvutil.sh.
getHealthTest(){
    ReportHealthNormal
}


# Write to out file in json format
# call: qn_write header $output_file <resources/shares>
 #call: qn_write footer $output_file
 #call: qn_write network $output_file
 #call: qn_write middleline $output_file
 #call: qn_write element $output_file $counter $

qn_write(){
    local _QN_OUT_FILE=${2}
    case $1 in
        header )
        # Write out json header
            echo "{" > ${_QN_OUT_FILE}
            echo -e "\"${3}\":[" >> ${_QN_OUT_FILE}
        ;;
        element )
        local _name="${4}"
        local _size="${5}"
        local _free="${6}"


        if [ $3 -ne 0 ]; then
			echo -e ", " >> ${_QN_OUT_FILE}
		fi
		echo -e {\"name\":\"${_name}\", \"capacity\":$_size, \"free\":$_free, \"available\":$_free}  >> ${_QN_OUT_FILE}
        ;;
        network )
        echo -e {\"name\":\"Network\", \"alt\":\"\", \"path\":\"smb://\", \"icon\":\"NETWORK\", \"hideEmpty\":false} >> ${_QN_OUT_FILE}
        ;;
        footer )
        # Write out json end
            echo "]"  >> ${_QN_OUT_FILE}
            echo "}"  >> ${_QN_OUT_FILE}
        ;;
        predefined_shares )
cat << EOF > ${_QN_OUT_FILE}
{
    "shares":[
        {
            "collection":"SMB"
        },
		{
            "name":"Network", "alt":"", "path":"smb://", "icon":"NETWORK", "hideEmpty":false
		}
    ]
}
EOF
        ;;
        *)
        echo "Incorrect usege in ${FUNCNAME}" >&2
        exit 1
        ;;
    esac
}

# Get device name (if device is root name is "Root file system"), size and free space
# And writes it in resources.lst file
GetResourcesByDf() {
    qn_write header ${_QN_RES_FILE} resources
	local list=`cat /proc/mounts | grep -E '/dev/sd|/dev/mapper|/dev/disk' | awk '{ print $1}'`
    IFS=$'\n'
    i=0
    _usbN=1
    for item in $list
    do
		device=$(df -B 1 | grep $item)
		path=`echo $device | awk '{print $6}'`
		name=`basename ${path}`
		case ${name} in
			"/")
				name="Root file system"
			;;
			"boot")
				continue
			;;
		esac
		size=$(echo $device | awk '{print $2}')
		free=$(echo $device | awk '{print $4}')
		qn_write element ${_QN_RES_FILE} $i $name $size $free

        i=$((i+1))
    done
    qn_write footer ${_QN_RES_FILE}
}

# find all the shared folders in those volumes, excluding folders with @
# For ubunt trusty and xenial we  can find samba shares in /var/lib/samba/usershares folder
# For Fedora no easy way to get samba shares. so may be we have to add predefined shareds
GetSharedFolders() {
    qn_write header ${_QN_SHARES_FILE} shares
    # Write user home folder
    if [ "$2" == "pc" ] || [ -z "$2" ]; then
		echo -e {\"name\":\"Home\", \"alt\":\".%name\'s Home\", \"path\":\"%home\", \"icon\":\"HOME\", \"hideEmpty\":false}, >> ${_QN_SHARES_FILE}
    fi
    OLDIFS=$IFS
    IFS=$'\n'

    # SHARES=`find $(ls -d /media) -maxdepth 2 -mindepth 2 -type d ! -path '*/.*'`
    SHARES=`cat /proc/mounts | grep -E '/dev/sd|/dev/mapper|/dev/disk|run/media' | awk '{ print $2}'`
    # First add local smb shares.
    # Connected disks
    # iterate through the shared folders and spit them out
    for d in ${SHARES}
        do
            name=`basename ${d}`
            case ${d} in
                "/")
                    continue
                    # name="Root file system"
                ;;
                "boot")
                    continue
                ;;
                /run/media/*)
                    :
                ;;
                *)
                    continue
                ;;
            esac
            # if [[ "$name" == "/" ]];then
            #	name="Root file system"
            # fi
            echo -e {\"name\":\"$name\", \"alt\":\"\", \"path\":\""${d}"\", \"icon\":\"ROOT\", \"hideEmpty\":false}, >> ${_QN_SHARES_FILE}
    done
    IFS=$OLDIFS

    qn_write network ${_QN_SHARES_FILE}
    qn_write footer ${_QN_SHARES_FILE}
}
