/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 6.3.2018
 * Time: 16:54
 * Author Petar Vanchev (petarv@qnext.com)
 * Author Milen Dimitrov (milend@qnext.com)
 */

/**
 *
 * @param {jQuery} $container
 * @constructor
 */
var CompProxySettings = function ($container) {

    BaseComp.call(this, $container);

    this._initContent();
};

QnUtil.inherits(CompProxySettings, BaseComp);
QnUtil.extend(CompProxySettings.prototype, {

    /* Public method */

    updatePanel: function () {
        var pConf = getGlobal(KEY_GLOBAL_PROXY).getConfigType();
        $('input[name=proxy_config]').each(function () {
            if ($(this).val() === pConf)
                $(this).trigger("click");
        });

        var isManual = $('input[name=proxy_config]:checked').val() === "Manual";
        var $comp = this._getCont().find("#CompProxySetting");
        this._renderProxyConfig($comp, !isManual);

        setTextContext("field_proxy_config_info");
        // add focus handler
        $comp.on("focus", "input", function () {
            switchTT($comp.parent());
        });
        // add hover handler
        $comp.hover(function () {
            switchTT($comp.parent());
        }, function () {
            $("body").find(".tooltipCont").css("display", "none");
            $(':focus').focus();
        });
    },

    /**
     * Get current proxy configuration from GUI
     *
     * @return {ProxyConfig} always valid instance. Will return proxy config get from list globals if no change are made
     */
    getProxyConfig: function () {
        var pConf = null;
        if (this.hasChanges()) {
            var confType = $('input[name=proxy_config]:checked').val();
            var isDetected = confType === ProxyConfig.CONFIG_TYPE_AUTO;
            var type, host, port, user, pass;

            if (confType === ProxyConfig.CONFIG_TYPE_MANUAL) {
                type = $('input[name=proxy_manual_config]:checked').val();
                var $cont = this._getCont();
                host = getComp("proxyHost", $cont).val().trim();
                port = parseInt(getComp("proxyPort", $cont).val().trim());
                user = getComp("proxyUsername", $cont).val().trim();
                pass = getComp("proxyPassword", $cont).val().trim();
            } else {
                type = ProxyConfig.TYPE_NOT_SET;
            }

            pConf = new ProxyConfig(type, isDetected, host, port, user, pass);
        } else {
            pConf = getGlobal(KEY_GLOBAL_PROXY);
        }

        return pConf;
    },

    /**
     * Check did GUI is changed from default values that was set on init
     *
     * @return {Boolean} always valid
     */
    hasChanges: function () {
        var hasChange = false;
        var pConf = getGlobal(KEY_GLOBAL_PROXY);
        var $cont = this._getCont();

        // check proxy configuration type
        var confType = $('input[name=proxy_config]:checked').val();
        if (confType !== pConf.getConfigType())
            hasChange = true;

        // check proxy manual configuration
        if (!hasChange && confType === ProxyConfig.CONFIG_TYPE_MANUAL) {
            var pType = $('input[name=proxy_manual_config]:checked').val();
            if (pType !== pConf.getType()) {
                hasChange = true;
            } else if (getComp("proxyHost", $cont).val().trim() !== pConf.getHost()) {
                hasChange = true;
            } else if (parseInt(getComp("proxyPort", $cont).val().trim()) !== pConf.getPort()) {
                hasChange = true;
            } else if (getComp("proxyUsername", $cont).val().trim() !== pConf.getUsername()) {
                hasChange = true;
            } else if (getComp("proxyPassword", $cont).val().trim() !== pConf.getPassword()) {
                hasChange = true;
            }
        }

        return hasChange;
    },


    /* Protected method */

    _initContent: function () {
        var $cont = this.createCont().attr("id", "CompProxySetting").appendTo(this._getCont());
        this._renderProxyChoice($cont);
        this._renderTooltip();

        var self = this;
        $("input[name=proxy_config]").change(function () {
            var isManual = $(this).val() === "Manual";
            self._renderProxyConfig(self._getCont().find("#CompProxySetting"), !isManual);
        });
    },


    /* Private method */

    /**
     * Add general tooltip for encryption comp that will be visible if any component is on focus or is hovered
     *
     * @private
     */
    _renderTooltip: function () {
        this._addTooltip(this._getCont(), "ProxyConfig", "field_proxy_config");
    },

    /**
     * Renders/adds new input of type radio with given "id" and given value
     * and a label (span). String UID is created using "field_"+[id].
     *
     * @param {jQuery} $cont container where to add new input element
     * @param {string} id component ID to be set as "id" attribute.
     * Used to create string too.
     * @param {string} value value of the input. For example "None".
     * @private
     */
    _renderRadioType: function ($cont, id, value) {
        $('<input type="radio" name="proxy_config" id="' + id +
            '" class="form-control field_proxy_config_input" value="' + value + '">')
            .appendTo($cont);
        $('<span class="field_proxy_config_label"></span>')
            .setTextById("field_" + id).appendTo($cont);
        $('<br/>').appendTo($cont);
    },

    /**
     * Render proxy choice and depend of current selection (choice) will render
     * another part of proxy config {@link #_renderProxyConfig}
     *
     * @param {jQuery} $cont
     * @private
     */
    _renderProxyChoice: function ($cont) {
        var $compCont = this.createCont().addClass("proxyChoice").appendTo($cont);
        var $comp = $('<div class="form-group"></div>');
        this._renderRadioType($comp, "proxy_config_none", ProxyConfig.CONFIG_TYPE_NONE);
        this._renderRadioType($comp, "proxy_config_auto", ProxyConfig.CONFIG_TYPE_AUTO);
        this._renderRadioType($comp, "proxy_config_manual", ProxyConfig.CONFIG_TYPE_MANUAL);
        $comp.appendTo($compCont);
    },

    /**
     * Render proxy configuration
     *
     * @param {jQuery} $cont
     * @param {Boolean} [isRemove]
     * @private
     */
    _renderProxyConfig: function ($cont, isRemove) {
        var $compCont = this._getCont().find('.proxyManualConfig');
        $compCont.remove();

        if (!isRemove) {
            $compCont = this.createCont().addClass("proxyManualConfig").appendTo($cont);
            var $row = this.createCont().addClass("form-group configRow").appendTo($compCont);
            this.createCont()
                .addClass("control-label")
                .setTextById("field_proxy_manual_config_type_header")
                .appendTo($row);
            $('<input type="radio" name="proxy_manual_config" id="proxy_manual_config_http" value="' + ProxyConfig.TYPE_HTTP + '" />')
                .addClass("form-control field_proxy_manual_config_input")
                .appendTo($row);
            $('<span></span>')
                .setTextById("field_proxy_manual_config_http")
                .addClass("field_proxy_manual_config_label")
                .appendTo($row);
            // $('<br/>').appendTo($row);
            $('<input type="radio" name="proxy_manual_config" id="proxy_manual_config_socks_v4" value="' + ProxyConfig.TYPE_SOCKS_4 + '" />')
                .addClass("form-control field_proxy_manual_config_input")
                .appendTo($row);
            $('<span></span>')
                .setTextById("field_proxy_manual_config_socks_v4")
                .addClass("field_proxy_manual_config_label")
                .appendTo($row);
            // $('<br/>').appendTo($row);
            $('<input type="radio" name="proxy_manual_config" id="proxy_manual_config_socks_v5" value="' + ProxyConfig.TYPE_SOCKS_5 + '" />')
                .addClass("form-control field_proxy_manual_config_input").appendTo($row);
            $('<span></span>')
                .setTextById("field_proxy_manual_config_socks_v5")
                .addClass("field_proxy_manual_config_label")
                .appendTo($row);
            // $('<br/>').appendTo($row);

            $row = this.createCont().addClass("form-group configRow").appendTo($compCont);
            this.createCont().addClass("control-label")
                .setTextById("field_proxy_manual_config_host_header")
                .appendTo($row);
            $('<input class="form-control proxyHost" required />')
                .setTooltip("field_proxy_manual_config_host_placeholder")
                .appendTo($row);

            $row = this.createCont().addClass("form-group configRow").appendTo($compCont);
            this.createCont()
                .addClass("control-label")
                .setTextById("field_proxy_manual_config_port_header")
                .appendTo($row);
            $('<input class="form-control proxyPort" required />')
                .setTooltip("field_proxy_manual_config_port_placeholder")
                .appendTo($row);

            $row = this.createCont().addClass("form-group configRow").appendTo($compCont);
            this.createCont()
                .addClass("control-label")
                .setTextById("field_proxy_manual_config_username_header")
                .appendTo($row);
            $('<input class="form-control proxyUsername" />')
                .setTooltip("field_proxy_manual_config_username_placeholder")
                .appendTo($row);

            $row = this.createCont().addClass("form-group configRow").appendTo($compCont);
            this.createCont()
                .addClass("control-label")
                .setTextById("field_proxy_manual_config_password_header")
                .appendTo($row);
            $('<input class="form-control proxyPassword" type="password" />')
                .setTooltip("field_proxy_manual_config_password_placeholder").appendTo($row);

            // add change listener to proxy type radio buttons
            var self = this;
            $('input[name=proxy_manual_config]').change(function () {
                // update port placeholder
                var curr = $(this).val();
                var labelStrId;
                switch (curr) {
                    case ProxyConfig.TYPE_HTTP:
                        labelStrId = "field_proxy_manual_config_http_port_placeholder";
                        break;
                    case ProxyConfig.TYPE_SOCKS_4:
                    case ProxyConfig.TYPE_SOCKS_5:
                        labelStrId = "field_proxy_manual_config_socks_port_placeholder";
                        break;
                    default:
                        labelStrId = "field_proxy_manual_config_port_placeholder";
                        break;

                }

                var $label = getComp("proxyPort", self._getCont());
                $label.setTooltip(labelStrId);
            });

            this._updateManualConfig();
        }
    },

    /**
     * Update manual proxy section
     *
     * @private
     */
    _updateManualConfig: function () {
        var proxyConfig = getGlobal(KEY_GLOBAL_PROXY);
        var pType = proxyConfig.isConfigManual() ? proxyConfig.getType() : ProxyConfig.TYPE_HTTP;
        // update proxy type
        $('input[name=proxy_manual_config]').each(function () {
            if ($(this).val() === pType)
                $(this).trigger("click");
        });

        var $cont = this._getCont();
        var host = proxyConfig.getHost();
        if (host)
            getComp("proxyHost", $cont).val(host);
        var port = proxyConfig.getPort();
        if (port && !isNaN(port))
            getComp("proxyPort", $cont).val(port);
        var user = proxyConfig.getUsername();
        if (user)
            getComp("proxyUsername", $cont).val(user);
    }

});
