/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 9.2.2018
 * Time: 16:19
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 * User settings frame
 *
 * @constructor
 */
var FrameUserSettings = function () {
    FrameEncrypt.call(this, FrameUserSettings.VID, $("body"));
};

QnUtil.inherits(FrameUserSettings, FrameEncrypt);
QnUtil.extend(FrameUserSettings.prototype, {

    /* Public method */

    updateFrame: function () {
        FrameUserSettings.superclass.updateFrame.apply(this);

        // enter a username into the edit box
        var username = getGlobal(KEY_GLOBAL_USERNAME);
        if (username) {
            this._getFormControl(FrameWithForm.FIELD_USERNAME).val(username);
        }

        // enter a domain into the edit box
        var domain = getGlobal(KEY_GLOBAL_DOMAIN);
        if (domain) {
            this._getFormControl(FrameWithForm.FIELD_DOMAIN).val(domain);
        }
    },


    /* Protected method */

    /**
     * @inheritDoc
     * @protected
     */
    _initContentImpl: function ($cont) {
        document.title = QnLocale.getStr("user_settings_app_title");
        var  $formCont = $('<div id="pageUserSettings"></div>').appendTo($cont);
        FrameUserSettings.superclass._initContentImpl.call(this, $formCont);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderPageTitle: function ($cont) {
        $('<h2 class="secureonly"></h2>')
            .setTextById("user_settings_title")
            .appendTo($cont);
        $('<p class="p secureonly"><span></span></p>')
            .appendTo($cont)
            .find("span")
            .setTextById("user_settings_subtitle");
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getBtnIdList: function () {
        return [BTN_SAVE_ID, BTN_CANCEL_ID];
    },

    /**
     *
     * @inheritDoc
     * @private
     */
    _checkCanSubmitImpl: function () {
        if(this._hasEncComp())
            return this._compEncryption.checkEncryption();

        return FrameUserSettings.superclass._checkCanSubmitImpl.apply(this);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _fillSubmitParams: function (params) {
        FrameUserSettings.superclass._fillSubmitParams.call(this, params);
        params["cmd"] = "update-share";
        params["device-id"] = getGlobal(KEY_GLOBAL_DEVICE_ID);
        params["device-password"] = this._getFormControl(FrameWithForm.FIELD_PASSWORD).val().trim();
        params["device-domain"] = this._getFormControl(FrameWithForm.FIELD_DOMAIN).val().trim();
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitSuccessImpl: function (resp, requestParams) {
        if (resp.result) {
            var redirectURL = resp["redirectURL"];
            log("User settings update successful");
            // indicate success visually
            showMessage(ERROR_OK, "settings_success");

            // if a redirect URL was provided by passive (via the server) we use it to send the user back immediately
            if (redirectURL != null) {
                // if there is a redirect URL, we immediately redirect the user
                log("redirectURL found: " + redirectURL);
                window.location.href = redirectURL;
            } else {
                log("no redirect URL was found");
            }
        } else {
            log("User settings update failure");
            if(this._hasEncComp())
                this._compEncryption.updateEncComps();
        }
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitFailImpl: function (xhr, err) {
        if (xhr.responseText) {
            try {
                var response = JSON.parse(xhr.responseText);
                var error = response["error"];
                log("User settings update failed error code: " + error["code"]);
            } catch (e) {
                log("User settings update failed: " + e);
            }
        }
        if(this._hasEncComp())
            this._compEncryption.updateEncComps();
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _setControlsState: function (onDisable, notForSubmit) {
        FrameUserSettings.superclass._setControlsState.call(this, onDisable, notForSubmit);

        // update buttons
        var $btnSubmit =  this._getBtn(BTN_SAVE_ID);
        if(!notForSubmit)
            $btnSubmit.button(onDisable ? 'loading' : 'reset');
        $btnSubmit.attr("disabled", onDisable);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _isReadOnly: function (fieldId) {
        switch (fieldId) {
            case FrameWithForm.FIELD_USERNAME:
            case FrameWithForm.FIELD_DOMAIN:
                return true;
            default:
                return FrameUserSettings.superclass._isReadOnly.call(this, fieldId);
        }
    }


    /* Private method */

});

/**
 *
 * @type {string}
 * @constant
 */
FrameUserSettings.VID = "FrameUserSettings";